var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "moment", "@xmr-core/xmr-constants"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    const moment_1 = __importDefault(require("moment"));
    const xmr_constants_1 = require("@xmr-core/xmr-constants");
    // Copyright (c) 2014-2018, MyMonero.com
    //
    // All rights reserved.
    //
    // Redistribution and use in source and binary forms, with or without modification, are
    // permitted provided that the following conditions are met:
    //
    // 1. Redistributions of source code must retain the above copyright notice, this list of
    //	conditions and the following disclaimer.
    //
    // 2. Redistributions in binary form must reproduce the above copyright notice, this list
    //	of conditions and the following disclaimer in the documentation and/or other
    //	materials provided with the distribution.
    //
    // 3. Neither the name of the copyright holder nor the names of its contributors may be
    //	used to endorse or promote products derived from this software without specific
    //	prior written permission.
    //
    // THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
    // EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
    // MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
    // THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
    // SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
    // PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
    // INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
    // STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
    // THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    function isTransactionConfirmed(tx, blockchainHeight) {
        return blockchainHeight - tx.height > xmr_constants_1.config.txMinConfirms;
    }
    exports.isTransactionConfirmed = isTransactionConfirmed;
    function isTransactionUnlocked({ unlock_time }, blockchainHeight) {
        if (!xmr_constants_1.config.maxBlockNumber) {
            throw Error("Max block number is not set in config!");
        }
        if (unlock_time < xmr_constants_1.config.maxBlockNumber) {
            // unlock time is block height
            return blockchainHeight >= unlock_time;
        }
        else {
            // unlock time is timestamp
            const current_time = Math.round(new Date().getTime() / 1000);
            return current_time >= unlock_time;
        }
    }
    exports.isTransactionUnlocked = isTransactionUnlocked;
    function transactionLockedReason({ unlock_time }, blockchainHeight) {
        if (unlock_time < xmr_constants_1.config.maxBlockNumber) {
            // unlock time is block height
            const numBlocks = unlock_time - blockchainHeight;
            if (numBlocks <= 0) {
                return "Transaction is unlocked";
            }
            const unlock_prediction = moment_1.default().add(numBlocks * xmr_constants_1.config.avgBlockTime, "seconds");
            return ("Will be unlocked in " +
                numBlocks +
                " blocks, ~" +
                unlock_prediction.fromNow(true) +
                ", " +
                unlock_prediction.calendar() +
                "");
        }
        else {
            // unlock time is timestamp
            const current_time = Math.round(new Date().getTime() / 1000);
            const time_difference = unlock_time - current_time;
            if (time_difference <= 0) {
                return "Transaction is unlocked";
            }
            const unlock_moment = moment_1.default(unlock_time * 1000);
            return ("Will be unlocked " +
                unlock_moment.fromNow() +
                ", " +
                unlock_moment.calendar());
        }
    }
    exports.transactionLockedReason = transactionLockedReason;
});
//# sourceMappingURL=tx-status.js.map